"""
Test the get_varname function
"""

import stldim

def test_empty_varname():
    """
    Test with an empty varnames
    """
    assert stldim.get_varname("test.stl", "") == "test_stl"

def test_none_varname():
    """
    Test with varname set to None
    """
    assert stldim.get_varname("test.stl", None) == "test_stl"

def test_varname():
    """
    Test with a varname set
    """
    assert stldim.get_varname("test.stl", "foobar") == "foobar"



"""
Tests for the sanitize_filename function in the stldim module.
"""

import stldim

def test_plain():
    """
    Test with a plain filename
    """
    print(dir(stldim))
    assert stldim.sanitize_filename("test.stl") == "test_stl"

def test_spaces():
    """
    Test with spaces in the filename
    """
    assert stldim.sanitize_filename("test test.stl") == "test_test_stl"

def test_special_chars():
    """
    Test with special characters in the filename
    """
    assert stldim.sanitize_filename("test!@#$%^&*().stl") == "test___________stl"

def test_leading_numbers():
    """
    Test with leading numbers in the filename
    """
    assert stldim.sanitize_filename("11test.stl") == "__test_stl"

def test_trailing_numbers():
    """
    Test with trailing numbers in the filename
    """
    assert stldim.sanitize_filename("test11.stl") == "test11_stl"

def test_no_extension():
    """
    Test with a filename with no extension
    """
    assert stldim.sanitize_filename("test") == "test"

def test_subdirectory():
    """
    Test with a filename with a subdirectory
    """
    assert stldim.sanitize_filename("subdir/test.stl") == "test_stl"



"""
COntains version information for the package
"""

__author__ = "nomike Postmann"
__copyright__ = "Copyright 2024 nomike Postmann"
__email__ = "nomike@nomike.com"
__license__ = "MIT"
__title__ = "stldim"
__version__ = "0.3.1"

__str__ = f'{__title__} v{__version__} by {__author__} <{__email__}>'



#!/usr/bin/env python3
"""
    stldim - Get dimensions of an STL file
    Usage:
        stldim.py [options] <stlfile>

    Options:
        -h --help       Show this screen.
        --version       Show version.
        --name=<name>   Name of the object [defaults to the filename with non alpha-numeric\
characters replaced with underscores].
"""

import os
import sys

from docopt import docopt
import jinja2

from stldim import MeshWithBounds, get_varname, version

def generate_openscad_lib(stl_dimensions, varname, stlfile):
    """
    Generate an OpenSCAD library file with the dimensions of the STL file
    """
    environment = jinja2.Environment(loader=jinja2.PackageLoader('stldim', 'templates'))
    template = environment.get_template('openscad_lib.jinja2')
    print(template.render(stl_dimensions=stl_dimensions, varname=varname, stlfile=stlfile))

def main():
    """
    Main function
    """
    args = docopt(__doc__, version=version.__str__)

    if not os.path.exists(args['<stlfile>']):
        sys.exit(f"ERROR: file {args['<stlfile>']} was not found!")
    varname = get_varname(args['<stlfile>'], args['--name'])

    stl_dimensions = MeshWithBounds.from_file(args['<stlfile>'])

    generate_openscad_lib(stl_dimensions, varname, args['<stlfile>'])


if __name__ == '__main__':
    main()



"""
A module that provides a class that extends the stl.Mesh class to include properties for the mesh's
dimensions.
"""

import os
import re

import stl
from stl import mesh


class MeshWithBounds(mesh.Mesh):
    """
    A class that extends the stl.Mesh class to include properties for the mesh's dimensions.
    """

    @property
    def maxx(self):
        """Calculate and return the maximum x of the mesh."""
        return max(self.points, key=lambda p: p[stl.Dimension.X.value])[stl.Dimension.X.value]

    @property
    def minx(self):
        """Calculate and return the minimum x of the mesh."""
        return min(self.points, key=lambda p: p[stl.Dimension.X.value])[stl.Dimension.X.value]

    @property
    def maxy(self):
        """Calculate and return the maximum y of the mesh."""
        return max(self.points, key=lambda p: p[stl.Dimension.Y.value])[stl.Dimension.Y.value]

    @property
    def miny(self):
        """Calculate and return the minimum y of the mesh."""
        return min(self.points, key=lambda p: p[stl.Dimension.Y.value])[stl.Dimension.Y.value]

    @property
    def maxz(self):
        """Calculate and return the maximum z of the mesh."""
        return max(self.points, key=lambda p: p[stl.Dimension.Z.value])[stl.Dimension.Z.value]

    @property
    def minz(self):
        """Calculate and return the minimum z of the mesh."""
        return min(self.points, key=lambda p: p[stl.Dimension.Z.value])[stl.Dimension.Z.value]

    @property
    def xsize(self):
        """Calculate and return the size of the mesh in the x direction."""
        return round(self.maxx - self.minx, 3)

    @property
    def ysize(self):
        """Calculate and return the size of the mesh in the y direction."""
        return round(self.maxy - self.miny, 3)

    @property
    def zsize(self):
        """Calculate and return the size of the mesh in the z direction."""
        return round(self.maxz - self.minz, 3)

    @property
    def midx(self):
        """Calculate and return the midpoint of the mesh in the x direction."""
        return round(self.xsize / 2, 3)

    @property
    def midy(self):
        """Calculate and return the midpoint of the mesh in the y direction."""
        return round(self.ysize / 2, 3)

    @property
    def midz(self):
        """Calculate and return the midpoint of the mesh in the z direction."""
        return round(self.zsize / 2, 3)

    @property
    def dimensions(self):
        """Return a dictionary of the mesh's dimensions."""
        return {
            'min_x': round(self.minx, 3),
            'max_x': round(self.maxx, 3),
            'min_y': round(self.miny, 3),
            'max_y': round(self.maxy, 3),
            'min_z': round(self.minz, 3),
            'max_z': round(self.maxz, 3),
            'x_size': round(self.xsize, 3),
            'y_size': round(self.ysize, 3),
            'z_size': round(self.zsize, 3),
            'mid_x': round(self.midx, 3),
            'mid_y': round(self.midy, 3),
            'mid_z': round(self.midz, 3),
        }

    def __getitem__(self, key):
        """
        Allow the object to be accessed like a dictionary.
        """
        return getattr(self, key)

def sanitize_filename(stlfile):
    """
    Replace every non-alphanumeric character with an underscore
    """

    sanitized = re.sub(r'\W', '_', os.path.basename(stlfile)).lower()
    match = re.search(r"\D", sanitized)
    if match:
        return "_" * (match.start()) + sanitized[match.start():]
    return sanitized


def get_varname(filename, name):
    """
    Return a sanitized variable name based on the filename and the provided name.
    """
    if name:
        return name
    return sanitize_filename(filename)
